package com.rokid.alliancedemo;

import androidx.appcompat.app.AppCompatActivity;

import android.hardware.usb.UsbDevice;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.widget.Toast;

import com.jiangdg.usbcamera.RKGlassDevice;
import com.jiangdg.usbcamera.UVCCameraHelper;
import com.jiangdg.usbcamera.callback.OnGlassConnectListener;
import com.rokid.alliance.base.hw.GlassInfo;
import com.serenegiant.usb.common.AbstractUVCCameraHandler;
import com.serenegiant.usb.encoder.RecordParams;

public class DemoVideoActivity extends AppCompatActivity {

    private boolean isVideoStart = false;
    private boolean isGlassConnected = false;
    private View btTakePhoto;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_demo_video);

        RKGlassDevice.getInstance().initUsbDevice(this, findViewById(R.id.camera_view), new OnGlassConnectListener() {
            @Override
            public void onGlassConnected(UsbDevice usbDevice, GlassInfo glassInfo) {
                isGlassConnected = true;

            }

            @Override
            public void onGlassDisconnected() {
                isGlassConnected = false;
                if (isVideoStart) {
                    UVCCameraHelper.getInstance().stopPusher();
                    isVideoStart = false;
                }
            }
        });

        View btVideoStart = findViewById(R.id.bt_video_start);
        btVideoStart.setOnClickListener(v -> {
            if (!isGlassConnected) {
                btVideoStart.post(() -> {
                    Toast.makeText(btVideoStart.getContext(), "请先连接眼镜!!!", Toast.LENGTH_LONG).show();
                });
                return;
            }
            if (isVideoStart) return;
            isVideoStart = true;
            RecordParams params = new RecordParams();
            params.setAutoSave(true);
            params.setRecordDuration(5);//这里的单位是minutes，用于分段存储 这里表示5分钟一个视频
            params.setRecordPath("/sdcard/rokiddemo/video");
            params.setVoiceClose(false);//false表示同时录制视频和音频 true表示录制只有视频画面
            UVCCameraHelper.getInstance().startPusher(params, new AbstractUVCCameraHandler.OnEncodeResultListener() {
                @Override
                public void onEncodeResult(byte[] data, int offset, int length, long timestamp, int type) {
                    //这里可以对camera数据进行添加水印等操作
                }

                @Override
                public void onRecordResult(String videoPath) {
                    btVideoStart.post(() -> {
                        Toast.makeText(btVideoStart.getContext(), "录制视频文件保存在: " + videoPath, Toast.LENGTH_LONG).show();
                    });
                }
            });
        });

        findViewById(R.id.bt_video_stop).setOnClickListener(v -> {
            if (!isGlassConnected) {
                btVideoStart.post(() -> {
                    Toast.makeText(btVideoStart.getContext(), "请先连接眼镜!!!", Toast.LENGTH_LONG).show();
                });
                return;
            }
            if (!isVideoStart) return;
            UVCCameraHelper.getInstance().stopPusher();
            isVideoStart = false;
        });

        btTakePhoto = findViewById(R.id.bt_take_photo);
        btTakePhoto.setOnClickListener(v -> {
            takePhoto();
        });
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (isVideoStart) {
            UVCCameraHelper.getInstance().stopPusher();
            isVideoStart = false;
        }
        RKGlassDevice.getInstance().deInit();
    }

    private void takePhoto() {
        if (!isGlassConnected) {
            btTakePhoto.post(() -> {

                Toast.makeText(btTakePhoto.getContext(), "请先连接眼镜!!!", Toast.LENGTH_LONG).show();
            });
            return;
        }

        UVCCameraHelper.getInstance().capturePicture("/sdcard/rokiddemo/photodemo.jpg", new AbstractUVCCameraHandler.OnCaptureListener() {
            @Override
            public void onCaptureResult(String path) {
                btTakePhoto.post(() -> {

                    Toast.makeText(btTakePhoto.getContext(), "拍照保存路径: " + path, Toast.LENGTH_LONG).show();
                });
            }
        });
    }
}
