package com.rokid.alliancedemo;

import android.Manifest;
import android.app.AppOpsManager;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.provider.Settings;
import android.view.WindowManager;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import com.rokid.mcui.utils.Logger;

import java.util.ArrayList;
import java.util.List;

import static com.rokid.mcui.utils.ContextUtil.getApplicationContext;

/**
 * 首页activity 主要配置权限
 */
public class SimpleDemoActivity extends AppCompatActivity {

    private static final String[] REQUIRED_PERMISSION_LIST = new String[]{
            Manifest.permission.WRITE_EXTERNAL_STORAGE,
            Manifest.permission.READ_EXTERNAL_STORAGE,
            Manifest.permission.RECORD_AUDIO,
            Manifest.permission.CAMERA,
            Manifest.permission.READ_PHONE_STATE,
    };
    private static final int REQUEST_CODE = 1;
    private List<String> mMissPermissions = new ArrayList<>(12);

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_simple_demo);

        initView();
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON, WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        //权限配置
        if (isVersionM() && !checkAndRequestPermissions()) {
            return;
        }



    }


    @Override
    protected void onDestroy() {
        super.onDestroy();
        getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
    }

    private boolean isVersionM() {
        return Build.VERSION.SDK_INT >= Build.VERSION_CODES.M;
    }
    private static final int ACTION_MANAGE_OVERLAY_PERMISSION_REQUEST_CODE = 2;

    private boolean checkAndRequestPermissions() {
//        Intent intent = new Intent(Settings.ACTION_MANAGE_OVERLAY_PERMISSION);
//        intent.setData(Uri.parse("package:" + getPackageName()));
//        startActivityForResult(intent, ACTION_MANAGE_OVERLAY_PERMISSION_REQUEST_CODE);
        mMissPermissions.clear();
        for (String permission : REQUIRED_PERMISSION_LIST) {
            int result = ContextCompat.checkSelfPermission(this, permission);
            if (result != PackageManager.PERMISSION_GRANTED) {
                mMissPermissions.add(permission);
            }
        }
        // check permissions has granted
        if (mMissPermissions.isEmpty()) {
            return true;
        } else {
            ActivityCompat.requestPermissions(this,
                    mMissPermissions.toArray(new String[mMissPermissions.size()]),
                    REQUEST_CODE);
        }
        return false;
    }

    @Override
    public void onRequestPermissionsResult(final int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        Logger.d("onRequestPermissionsResult-------->requestCode = " + requestCode);
        if (requestCode == REQUEST_CODE) {
            for (int i = grantResults.length - 1; i >= 0; i--) {
                if (grantResults[i] == PackageManager.PERMISSION_GRANTED) {
                    mMissPermissions.remove(permissions[i]);
                }
            }
        }
        // Get permissions success or not
        if (mMissPermissions.isEmpty()) {
        } else {
            for (String mMissPermission : mMissPermissions) {

                Logger.i("required permission : ", mMissPermission);
            }
            Toast.makeText(this, "权限问题", Toast.LENGTH_SHORT).show();
            this.finish();
        }
    }

//    @Override
//    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
//        super.onActivityResult(requestCode, resultCode, data);
//
//        if (requestCode == ACTION_MANAGE_OVERLAY_PERMISSION_REQUEST_CODE) {
//            // 权限授予后返回需要一定的延时才能检测到权限改变！！！！！
//            new Handler().postDelayed(new Runnable() {
//                @Override
//                public void run() {
//                    if (!hasOverlay(getApplicationContext())) {
//                        // You don't have permission
//                        Toast.makeText(getApplication(), "浮层权限", Toast.LENGTH_SHORT).show();
//                        finish();
//                    }
//                }
//            },1000);
//        }
//    }

    private boolean hasOverlay(Context context) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            // 在8.0上，有的手机是区分了mode的，导致使用Settings.canDrawOverlays(context) 方法判断的时候会出问题
            AppOpsManager appOpsMgr = (AppOpsManager) context.getSystemService(Context.APP_OPS_SERVICE);
            if (appOpsMgr == null){
                return false;
            }

            int mode ;
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q){
                mode = appOpsMgr.unsafeCheckOpNoThrow("android:system_alert_window", android.os.Process.myUid(), context.getPackageName());
            }else{
                mode = appOpsMgr.checkOpNoThrow("android:system_alert_window", android.os.Process.myUid(), context.getPackageName());
            }
//            AppOpsManager.MODE_ALLOWED = 0 —— 表示授予了权限并且重新打开了应用程序
//            AppOpsManager.MODE_IGNORED = 1 —— 忽略
//            AppOpsManager.MODE_ERRORED = 2 —— 表示当前应用没有此权限
//            AppOpsManager.MODE_DEFAULT = 3 —— 表示默认值，有些手机在没有开启权限时，mode的值就是这个
            // 在Android8.0中
            return Settings.canDrawOverlays(this) || mode == AppOpsManager.MODE_ALLOWED;
        }else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M ){
            return Settings.canDrawOverlays(this);
        }else{
            return ContextCompat.checkSelfPermission(this, Manifest.permission.SYSTEM_ALERT_WINDOW) == PackageManager.PERMISSION_GRANTED;
        }

    }
    private void initView() {


        findViewById(R.id.bt_presentation).setOnClickListener(v -> {
            startActivity(new Intent(this, DemoPresentationActivity.class));//双屏异显Demo
        });


        findViewById(R.id.bt_glass).setOnClickListener((v) -> {
            startActivity(new Intent(this, DemoGlassActivity.class));//按键、传感器、眼镜信息获取
        });


        findViewById(R.id.bt_audio_record).setOnClickListener( v -> {
            startActivity(new Intent(this, DemoAudioRecordActivity.class));
        });//录音Demo


        findViewById(R.id.bt_voice_command).setOnClickListener( v -> {
            startActivity(new Intent(this, RKVoiceCommandActivity.class));//Rokid离线语音Demo
        });

        findViewById(R.id.bt_video_record).setOnClickListener( v -> {
            startActivity(new Intent(this, DemoVideoActivity.class));//Rokid录制视频demo
        });
    }


}
